/*BEGIN OCO COPYRIGHT
 *************************************************************************
 *
 * Licensed Materials - Property of IBM
 * pidnumber 5724-L01,5724-I82,5655-N53,5655-R15
 * (C) Copyright IBM Corporation 2006.
 * US Government Users Restricted Rights- Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 **************************************************************************
 *END OCO COPYRIGHT
 */

package com.ibm.websphere.samples.databinding.util;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;

public class TypeHelper
{
    private static final TimeZone GMT_TIME_ZONE = TimeZone.getTimeZone("GMT"); //$NON-NLS-1$

    public TypeHelper()
    {
    }
    
    /**
     * Convert the value to a string
     * @param propertyType This provides the type of the value
     * @param value The actual value
     * @return String value of the value
     */
    public String convertToString(String propertyType,
                                         Object value)
    {
        int index = propertyType.lastIndexOf(".");
        propertyType = propertyType.substring(index + 1, propertyType.length());

        if (propertyType.equalsIgnoreCase("Date")) {
            return formatLocalXMLDate("yyyy-MM-dd", (Date) value); //$NON-NLS-1$
        }
        if (propertyType.equalsIgnoreCase("DateTime")) {
            return formatXMLDate("yyyy-MM-dd'T'HH:mm:ss.S'Z'", (Date) value); //$NON-NLS-1$
        }       
        if (value == null)
            return null;
        
        return value.toString();
    }


    private String formatLocalXMLDate(String pattern,
                                             Date source)
    {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern, Locale.US);
        String output = sdf.format(source);
        return output;
    }


    private String formatXMLDate(String pattern,
                                        Date source)
    {
        SimpleDateFormat sdf = new SimpleDateFormat(pattern, Locale.US);
        sdf.setTimeZone(GMT_TIME_ZONE);
        sdf.getCalendar().setTimeZone(GMT_TIME_ZONE);
        String output = sdf.format(source);
        return output;
    }


    /**
     * Converts a string value to the type given by propertyType
     * @param propertyType type of the resultant value
     * @param initialValue string that needs to be converted
     * @return converted value
     */
    public  Object createFromString(String propertyType,
                                   String initialValue)
    {
        int index = propertyType.lastIndexOf(".");
        propertyType = propertyType.substring(index + 1, propertyType.length());

        if (propertyType.equalsIgnoreCase("Boolean")) {
            return createBooleanFromString(initialValue);
        }
        if (propertyType.equalsIgnoreCase("Byte")) {
            return Byte.valueOf(initialValue);
        }
        if (propertyType.equalsIgnoreCase("Date")) {
            return createDateFromString("yyyy-MM-dd", initialValue);
        }
        if (propertyType.equalsIgnoreCase("DateTime")) {
            createDateTimeFromString("yyyy-MM-dd'T'HH:mm:ss.SSS", initialValue);
        }
        if (propertyType.equalsIgnoreCase("Decimal")) {
            return createDecimalFromString(initialValue);
        }
        if (propertyType.equalsIgnoreCase("Double")) {
            return createDoubleFromString(initialValue);
        }
        if (propertyType.equalsIgnoreCase("Float")) {
            return createFloatFromString(initialValue);
        }
        if (propertyType.equalsIgnoreCase("int")) {
            return createIntFromString(initialValue);
        }
        if (propertyType.equalsIgnoreCase("Integer")) {
            return createIntegerFromString(initialValue);
        }
        if (propertyType.equalsIgnoreCase("Long")) {
            return createLongFromString(initialValue);
        }
        return initialValue;

    }


    private  Date createDateFromString(String pattern,
                                      String value)
        throws IllegalArgumentException
    {
        if (value.endsWith("Z")) { //$NON-NLS-1$

            value = value.substring(0, value.length() - 1);
        }
        SimpleDateFormat sdf = new SimpleDateFormat(pattern, Locale.US);

        // parse the date time string.
        ParsePosition pos = new ParsePosition(0);
        Date result = sdf.parse(value, pos);
        if (pos.getIndex() == 0) {
            throw new IllegalArgumentException("Unparseable date: \"" + value + "\""); //$NON-NLS-1$  //$NON-NLS-2$
        }
        return result;
    }


    private  Boolean createBooleanFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return booleanValueOf(initialValue);
        }
    }


    protected  Boolean booleanValueOf(String initialValue)
    {
        if ("true".equals(initialValue) || "1".equals(initialValue)) //$NON-NLS-2$  //$NON-NLS-1$
        {
            return Boolean.TRUE;
        }
        if ("false".equals(initialValue) || "0".equals(initialValue)) //$NON-NLS-2$  //$NON-NLS-1$
        {
            return Boolean.FALSE;
        }
        return Boolean.FALSE;

    }


    private  Date createDateTimeFromString(String pattern,
                                          String source)
        throws IllegalArgumentException
    {
        if (source.endsWith("Z")) { //$NON-NLS-1$
            source = source.substring(0, source.length() - 1);
        }

        SimpleDateFormat sdf = new SimpleDateFormat(pattern, Locale.US);
        sdf.setTimeZone(GMT_TIME_ZONE); // set TimeZone to GMT
        ParsePosition pos = new ParsePosition(0);
        Date result = sdf.parse(source, pos);

        if (pos.getIndex() == 0) {
            throw new IllegalArgumentException("Unparseable date: \"" + source + "\""); //$NON-NLS-1$  //$NON-NLS-2$
        }
        return result;
    }


    private  BigDecimal createDecimalFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return new BigDecimal(initialValue);
        }
    }


    private  Double createDoubleFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return Double.valueOf(initialValue);
        }
    }


    private  Float createFloatFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return Float.valueOf(initialValue);
        }
    }


    private  Integer createIntFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return Integer.valueOf(initialValue);
        }
    }


    private  BigInteger createIntegerFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return new BigInteger(initialValue);
        }
    }


    private  Long createLongFromString(String initialValue)
    {
        if (initialValue == null) {
            return null;
        }
        else {
            return Long.valueOf(initialValue);
        }
    }
}
